	var map = null;
	var global_marker = null;
	var geocoder = null;
	var v_latitude = null;
	var v_longitude = null;
	var v_address = null;
	var v_address_object = {};
	var initialSetup = null;
	
	// private setMarker() - sets market to a particular Point
	function setMarker(point)
	{
		if (window.global_marker)
		{
			window.global_marker.setPosition(point);
		} else
		{
			window.global_marker = new google.maps.Marker({position: point, map: window.mapobject, draggable: true});
		}
		
		google.maps.event.addListener(global_marker,"dragend", 
			function(evt) 
			{ 
				if (evt.latLng)
				{
					v_latitude = evt.latLng.lat();
					v_longitude = evt.latLng.lng();
					updateCoords();
					
					// also update 
					getAddressForPoint(evt.latLng);
				}
			}
		);
	}
	
	// private updateCoords()  - creates a "change" event, so we can notify our parent listener that the focus of the Marker has changed
	function updateCoords()
	{
		// I am calling the current coordinates out to the parent window, which should
		// have an event handler ready
		
		//alert('dispatching event: ' + v_latitude + 'x' + v_longitude);
		
		var evt = document.createEvent("HTMLEvents");
			evt.initEvent("change", true, false);
			evt.lat = v_latitude;
			evt.lng = v_longitude;
			evt.address = v_address;
			evt.address_object = v_address_object;
			
			document.dispatchEvent(evt);
	}
	
	// public textSearch() -  will search for an addreess, focus the map on it and put a marker on that location
	function textSearch(myval)
	{
		if(myval)
		{
			if (geocoder) {
				
				geocoder.geocode(
					{
						address: myval
					},
					function(return_object, result_code)
					{
						if (result_code == google.maps.GeocoderStatus.OK)
						{
							if (return_object[0])
							{
								if (return_object[0].geometry.location)
								{
									window.mapobject.setCenter(return_object[0].geometry.location);
									window.mapobject.setZoom(13);
									
									setMarker(return_object[0].geometry.location);
									
									v_latitude = return_object[0].geometry.location.lat();
									v_longitude = return_object[0].geometry.location.lng();
									v_address = myval;
									
									updateCoords();
									
									
								}
							}
						} else
						{
							alert(myval + " not found");
						}
					}
				);
			}
		}
	}
	
	// public prefillSetup
	function prefillSetup(setup)
	{
		initialSetup = setup;
	}
	
	// public setCoords - focuses the map on the provided coords and puts a marker on the point
	function setCoords(lng, lat)
	{
		var point = new google.maps.LatLng(lat, lng);
		window.mapobject.setCenter(point);
		window.mapobject.setZoom(13);
		
		setMarker(point);
	}
	
	// searchLocation() - public wrapper for textSearch();
	function searchLocation(helpstring)
	{
		if (helpstring)
		{
			textSearch(helpstring);
		}
	}
	
	// private - getAddressForPoint sets global address string v_address and calls updateCoords from point location of type GLatLng
	function getAddressForPoint(point)
	{
		if (geocoder)
		{
			geocoder.geocode(
				{
					latLng: point
				},
				function(return_object, result_code)
				{
					if (result_code == google.maps.GeocoderStatus.OK)
					{
						if (return_object[0])
						{
							var addr_c = return_object[0].address_components;
							
							v_address = return_object[0].formatted_address;
								
							v_address_object.country_code = addr_c[4].short_name;
							v_address_object.county =  addr_c[2].long_name;
							v_address_object.city = addr_c[3].long_name;
							v_address_object.street = addr_c[1].long_name;
							v_address_object.street_num = addr_c[0].long_name;
							v_address_object.zip =  addr_c[5].long_name;
							
						}
						
						updateCoords();
					} else
					{
						// address not found yet
					}
				}
			);
		}
	}
	
	// private - init function
	function initialize() 
	{
		var centerpoint;
		var zoom = 3;
		var searchAtInit = false;
		
		if(initialSetup)
		{
			if (initialSetup.lng)
			{
				if (initialSetup.lat)
				{
					//setCoords(initialSetup.lng, initialSetup.lat); //this will fire itself a few rows later
					v_latitude = initialSetup.lat;
					v_longitude = initialSetup.lng;
				}
			}
			else
			{
				if (initialSetup.helpstring)
				{
					searchAtInit = true;
					//searchLocation(initialSetup.helpstring); //fire this after the map's been initialized
				}
			}
		}
		
		if (v_latitude && v_longitude)
		{
			centerpoint = new google.maps.LatLng(v_latitude, v_longitude);
			zoom = 13;
		}
		else
		{
			centerpoint = new google.maps.LatLng(49, 20);
		}
		
		var map_options = {
			zoom: zoom,
			center: centerpoint,
			mapTypeId: google.maps.MapTypeId.ROADMAP
		};
		
		window.mapobject = new google.maps.Map(document.getElementById("map_canvas"), map_options);
		
		//geocoder = new GClientGeocoder();
		geocoder = new google.maps.Geocoder();
		
		//window.mapobject.setUIToDefault();
		
		google.maps.event.addListener(window.mapobject,"dblclick", 
			function(mapevt) 
			{ 
				if (mapevt.latLng)
				{
					v_latitude = mapevt.latLng.lat();
					v_longitude = mapevt.latLng.lng();
					
					setMarker(mapevt.latLng);
					updateCoords();
				}
			}
		);
		
		if (searchAtInit)
		{
			//no point defined, autosearch for location
			searchLocation(initialSetup.helpstring);
		}
		else
		{
			//specific latlng or default center defined
			setMarker(centerpoint);
		}
	}
	
	var initializeTimer;
	
	function checkApi ()
	{
		if (typeof google != 'undefined')
		{
			console.log('google is already defined, initializing map ...');
			window.clearInterval(initializeTimer);
			initialize();
		}
		else
		{
			console.log('google still not ready ...');
		}
	}
	
	document.addEventListener(
		"DOMContentLoaded",
		function(event) {
			initializeTimer = window.setInterval(checkApi, 100);
		}
	);